% function to compute Statistically invariant features that are to be
% rather augmented 
% Input: 
% M = d X N vector time series, each of length N

% Output:
% sttf = [vector of statistical features]
function [indf, corf, sttf] = getSTATfeatures (M)

  [d , N] = size(M);
  % Computing statistically invariant features
    
  avg_ = mean(M,2);
  std_ = var(M,1,2);
  max_ = max(M,[],2);
  min_ = min(M,[],2);
  rng_ = max_ - min_;
  rms_ = sqrt(sum(M.^2,2)/N);
  rms_ = (rms_-min(rms_))./max(rms_);
  % skew mean and variance
  skm_ = mean((M-repmat(avg_, 1 ,N)).^3./repmat(std_, 1 ,N).^3, 2);
  skm_ = (skm_-min(skm_))./max(skm_);
  skv_ = var((M-repmat(avg_, 1 ,N)).^3./repmat(std_, 1 ,N).^3, 1, 2);
  skv_ = (skv_-min(skv_))./max(skv_);
  % Kurtosis mean and variance
  ksm_ = mean((M-repmat(avg_, 1 ,N)).^4./repmat(std_, 1 ,N).^4, 2);
  ksm_ = (ksm_-min(ksm_))./max(ksm_);
  ksv_ = var((M-repmat(avg_, 1 ,N)).^4./repmat(std_, 1 ,N).^4, 1, 2);
  ksv_ = (ksv_-min(ksv_))./max(ksv_);
  % Pairwise correlation as features
  XCORR = corr(M',M');
  cft_ = [];
  for i=1:d-1
    cft_ = [cft_; diag(XCORR, i)];    
  end
  cft_ = cft_-min(cft_);
  cft_ = cft_./max(cft_);
  % Concatenating all the above features to form a large vector
  indf = [avg_; std_; max_; min_; rng_; rms_; skm_; skv_; ksm_; ksv_]';
  corf = cft_';
  sttf = [indf, corf];
return
