% Script to perform event detection using discrete HMMs
path('./tools/HMM', path);
path('./tools/KPMtools', path);
path('./tools/KPMstats', path);
IN_DIR = './data';
file = 'EC.mat';
load([IN_DIR '/' file]);
labels = dlmread([IN_DIR '/EC-labels.txt'], '/');

% Cross validation splits
nsplits = 5;

% Discretize each sequence : Preparing data for Discrete HMM training
IDS = [];
DATA = [];
for i =1:size(TSdata,2)
  fprintf(1, 'Discretizing video id: %d\n', TSdata(i).id);
  if size(TSdata(i).seq, 2) > 5
     s = labels(labels(:,2)==TSdata(i).id, :);
     if ~isempty(s)
       [~, idx] = max(TSdata(i).seq);
       IDS = [IDS; s(1) s(2)];
       DATA = [DATA; {idx}];
     end
  end
end

labels = dlmread([IN_DIR '/EC2-labels.txt'], '/');
file = 'EC2.mat';
load([IN_DIR '/' file]);

for i =1:size(TSdata,2)
  fprintf(1, 'Discretizing video id: %d\n', TSdata(i).id);
  if size(TSdata(i).seq, 2) > 5
     s = labels(labels(:,2)==TSdata(i).id, :);
     if ~isempty(s)
       [~, idx] = max(TSdata(i).seq);
       IDS = [IDS; s(1) s(2)];
       DATA = [DATA; {idx}];
     end
  end
end
% Generating the splits
IDS = [IDS, mod(IDS(:,2), nsplits) + 1];

% Estimate the transmission and emission probabilities from the training
% data
MAX_ITER = 200;                  % maximum iteration
N_STATES = 5;                    % Number of states
cov_type = 'full';
N_SYMS = size(TSdata(1).seq, 1); % Number of symbols
prior0 = normalise(rand(N_STATES,1));
transmat0 = mk_stochastic(rand(N_STATES, N_STATES));
obsmat0 = mk_stochastic(rand(N_STATES, N_SYMS));

% Perform Classification for each split
classes = unique(sort(IDS(:,1)));
nclasses = size(classes,1);
model = struct();

RES = [];
for i=1:nsplits
  % Testing data for ith split
  tsDATA = DATA(IDS(:,3)==i,:);
  tsIDS  = IDS(IDS(:,3)==i,:);
  % Training data for ith split
  trDATA = DATA(IDS(:,3)~=i,:);
  trIDS  = IDS(IDS(:,3)~=i,:);
  
  % Perform training for each event model
  for j=1:nclasses
    cl_id = classes(j);
    fprintf(1, 'Split: %d __ Training HMM for model %d\n', i, cl_id);
    [~, model(cl_id).prior, model(cl_id).transmat, model(cl_id).obsmat] = ... 
         dhmm_em(trDATA(trIDS(:,1)==cl_id), prior0, ... 
         transmat0, obsmat0, 'max_iter', MAX_ITER);
    model(cl_id).obsmat(model(cl_id).obsmat==0) = 1e-6;
  end
  
  % Perform testing 
  r = [];
  for k=1:size(tsDATA,1)
    % Test for every learned model
    p = [];
    for j=1:nclasses
      cl_id = classes(j);
      p = [p, exp(dhmm_logprob(tsDATA(k), ... 
            model(cl_id).prior, model(cl_id).transmat, model(cl_id).obsmat))];
    end
    r = [r; p];
  end
  [~, idxs] = max(r');
  % Predicted Label, True Label, Video ID, SPLIT ID, Event Probabilities
  RES = [RES; idxs', tsIDS, r];
end

RES(RES(:,1)>15) = RES(RES(:,1)>15)+5;
% Save Data
save([IN_DIR '/HMM_EC-' file], 'RES');