function [prob_matrix confusion_matrix accuracy] = histogram_kfold_test(folds, usePredicted)
files = dir('./concept detector');
files = files(3:end);
fileCount = length(files);
load('cls_output_concepts_16384.mat');
actionsSize = length(class_map);
clear final_p final_acc c_acc class_map
%array that holds all the matrices for training data
training_arrays = cell(fileCount, 1);
%cell array that holds all the video sequence data
data_arrays = cell(fileCount, 2);

num_sequences = zeros(1, fileCount); %the number of video sequences in each event
for i=1:fileCount
    load(['./concept detector/' files(i).name]);
    data_arrays{i, 1} = {event_selector.predicted}';
    data_arrays{i, 2} = cell2mat({event_selector.video_id}');
    data_arrays{i, 3} = {event_selector.ground_truth}';
    num_sequences(i) = length(event_selector);
end

prob_matrix = zeros(sum(num_sequences), fileCount+3); %matrix to hold all the probabilities for each video over all events
prob_matrix(:, 4:end) = -1;
num_sequences = [0 cumsum(num_sequences)]; %this line makes it easier for indexing later

for i=1:fileCount%fill in the first two columns of prob_matrix
    video_id = data_arrays{i, 2};
    for j = fliplr(1:length(video_id))
       prob_matrix(num_sequences(i)+j, 1) = video_id(j); %video ID
       prob_matrix(num_sequences(i)+j, 2) = str2double(files(i).name(2:5)); %ground truth 
    end
end
prob_matrix(prob_matrix(:, 1) == -1, :) = []; %remove all the empty cells
testing_vector = cell(1, fileCount); %holds all the testing data across all events
indices = cell(fileCount, 1);
%randomize training and testing classification
for i=1:fileCount
    data = data_arrays{i, 1};
    indices{i} = crossvalind('Kfold', length(data),folds);
end
for fold=1:folds %iterate over all folds
    for i=1:fileCount %compute training data for each class
        if usePredicted
            training_data = data_arrays{i, 1}(indices{i}~=fold);
        else
            training_data = data_arrays{i, 3}(indices{i}~=fold);
        end
        training_arrays{i} = histogram(training_data, actionsSize);
        testing_vector{i} = find(indices{i} == fold);
    end
    
    for event_i = 1:fileCount %iterate over the testing data set across all events
       %bring up data and video_id arrays for given event
       data = data_arrays{event_i, 1};
       video_id = data_arrays{event_i, 2};
       for video_i = 1:length(testing_vector{event_i}); %iterate over all the test videos in a single event
            %bring up test_sequence and its row in the probability matrix
            test_sequence = data(testing_vector{event_i}(video_i));
            test_sequence = test_sequence{1};
            row = prob_matrix(:,1) == video_id(testing_vector{event_i}(video_i));
            for model = 1:fileCount %iterate over all histogram vectors for each video
                wrapper = {test_sequence};
                test_histogram = histogram(wrapper, actionsSize);
                prob_matrix(row, model+3) = sum(test_histogram(:).*training_arrays{model}(:));
            end
           if all(~prob_matrix(row, 4:end)) %check if all the dot products are zero
               prob_matrix(row, 3) = 0;
           else
                [~, index] = max(prob_matrix(row, 4:end));
                prob_matrix(row, 3) = str2double(files(index).name(2:5));
           end
       end
    end
end
nonzero_rows = prob_matrix(:, 3) ~= 0;
ground_truth = prob_matrix(:, 2);
predicted = prob_matrix(:, 3);
confusion_matrix = confusionmat(ground_truth, predicted);
row_sum = sum(confusion_matrix, 2);
for i = 1:length(confusion_matrix) %normalize rows
    if row_sum(i) ~= 0    
       confusion_matrix(i, :) = confusion_matrix(i, :)/row_sum(i);
    end
end
%conf_mat = conf_mat/folds;
num_correct = sum(prob_matrix(:, 2) == prob_matrix(:, 3));
accuracy = num_correct / length(prob_matrix);